@@ -1,4 +1,4 @@
-/* $Id: gucharmap-script-chapters.c 1358 2007-09-05 06:58:05Z behdad $ */
+/* $Id: gucharmap-script-chapters.c 1376 2007-12-03 21:42:48Z chpe $ */
 /*
  * Copyright (c) 2004 Noah Levitt
  *
@@ -26,6 +26,7 @@
 #include "gucharmap-script-chapters.h"
 #include "gucharmap-script-codepoint-list.h"
 #include "gucharmap-chapters.h"
+#include "gucharmap-settings.h"
 
 enum 
 {
@@ -40,11 +41,34 @@ selection_changed (GtkTreeSelection        *selection,
 {
   GtkTreeModel *model;
   GtkTreeIter iter;
+  gchar *chapter = NULL;
 
   /* this should always return true */
   if (gtk_tree_selection_get_selected (selection, &model, &iter))
     /* XXX: parent should have api to do this I guess */
     g_signal_emit_by_name (GUCHARMAP_CHAPTERS (chapters), "changed", 0); 
+
+  if ((chapter = gucharmap_chapter_get_string (GUCHARMAP_CHAPTERS (chapters))))
+    {
+      gucharmap_settings_set_chapter (chapter);
+      g_free (chapter);
+    }
+}
+
+static void
+gucharmap_script_chapters_settings_init (GucharmapChapters *chapters)
+{
+  GtkTreeSelection *sel;
+  gchar *chapter = NULL;
+
+  sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (chapters->tree_view));
+
+  chapter = gucharmap_settings_get_chapter ();
+  if (chapter)
+    gucharmap_chapter_set_string (chapters, chapter);
+  g_free (chapter);
+
+  gucharmap_settings_set_chapters_mode (CHAPTERS_SCRIPT);
 }
 
 static gunichar
@@ -153,6 +177,8 @@ gucharmap_script_chapters_init (GucharmapScriptChapters *chapters)
 
   queue_select_default_script (chapters);
 
+  gucharmap_script_chapters_settings_init (parent);
+
   gtk_container_add (GTK_CONTAINER (chapters), parent->tree_view);
   gtk_widget_show (parent->tree_view);
 }
