@@ -1,4 +1,4 @@
-/* $Id: gucharmap-window.c 1300 2007-01-19 18:32:05Z chpe $ */
+/* $Id: gucharmap-window.c 1382 2007-12-03 22:40:55Z chpe $ */
 /*
  * Copyright (c) 2004 Noah Levitt
  *
@@ -24,6 +24,7 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #ifdef HAVE_GNOME
+# include <gconf/gconf-client.h>
 # include <gnome.h>
 #endif
 #include "gucharmap-window.h"
@@ -33,6 +34,7 @@
 #include "gucharmap-block-chapters.h"
 #include "gucharmap-intl.h"
 #include "gucharmap-search-dialog.h"
+#include "gucharmap-settings.h"
 
 #ifndef ICON_PATH
 # define ICON_PATH ""
@@ -41,13 +43,6 @@
 #define GUCHARMAP_WINDOW_GET_PRIVATE(o) \
             (G_TYPE_INSTANCE_GET_PRIVATE ((o), gucharmap_window_get_type (), GucharmapWindowPrivate))
 
-typedef enum
-{
-  CHAPTERS_SCRIPT,
-  CHAPTERS_BLOCK
-}
-ChaptersMode;
-
 typedef struct _GucharmapWindowPrivate GucharmapWindowPrivate;
 
 struct _GucharmapWindowPrivate
@@ -290,17 +285,18 @@ font_default (GtkAction       *action,
 }
 
 static void
-snap_cols_pow2 (GtkAction 	 *action, 
+snap_cols_pow2 (GtkAction        *action, 
                 GucharmapWindow  *guw)
 {
-  gucharmap_table_set_snap_pow2 (guw->charmap->chartable,
-  				 gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)));
+  gboolean is_active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+  gucharmap_table_set_snap_pow2 (guw->charmap->chartable, is_active);
+  gucharmap_settings_set_snap_pow2 (is_active);
 }
 
 #ifdef HAVE_GNOME
 static void
 help_contents (GtkAction *action,
-	       gpointer  data)
+               gpointer  data)
 {
   GError *error = NULL;
 
@@ -591,7 +587,7 @@ make_menu (GucharmapWindow *guw)
   g_object_unref (priv->accel_group);
 
   toggle_menu = gtk_toggle_action_new ( "SnapColumns",
-					_("Snap Columns to Power of Two"),
+					_("Snap _Columns to Power of Two"),
 					NULL,
 					NULL);
   g_signal_connect (G_OBJECT(toggle_menu), "toggled", G_CALLBACK (snap_cols_pow2), guw);
@@ -612,12 +608,14 @@ make_menu (GucharmapWindow *guw)
   gtk_action_group_add_radio_actions (priv->action_group,
   				      radio_menu_entries,
 				      G_N_ELEMENTS (radio_menu_entries),
-				      VIEW_BY_SCRIPT,
+				      gucharmap_settings_get_chapters_mode(),
 				      G_CALLBACK (view_by),
 				      guw);
 
   gtk_action_group_add_action (priv->action_group,
   			       GTK_ACTION (toggle_menu));
+  gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (toggle_menu),
+                                gucharmap_settings_get_snap_pow2 ());
   switch (priv->chapters_mode)
     {
       case CHAPTERS_SCRIPT:
@@ -665,6 +663,7 @@ fontsel_changed (GucharmapMiniFontSelection *fontsel,
   gchar *font_name = gucharmap_mini_font_selection_get_font_name (fontsel);
 
   gucharmap_table_set_font (guw->charmap->chartable, font_name);
+  gucharmap_settings_set_font (font_name);
 
   g_free (font_name);
 }
@@ -718,9 +717,6 @@ make_text_to_copy (GucharmapWindow *guw)
   GtkWidget *button;
   GtkWidget *label;
   GtkWidget *hbox;
-  GtkTooltips *tooltips;
-
-  tooltips = gtk_tooltips_new ();
 
   hbox = gtk_hbox_new (FALSE, 6);
 
@@ -745,7 +741,7 @@ make_text_to_copy (GucharmapWindow *guw)
   g_signal_connect (G_OBJECT (priv->text_to_copy_entry), "changed",
                     G_CALLBACK (entry_changed_sensitize_button), button);
 
-  gtk_tooltips_set_tip (tooltips, button, _("Copy to the clipboard."), NULL);
+  gtk_widget_set_tooltip_text (button, _("Copy to the clipboard."));
 
   return hbox;
 }
@@ -839,7 +835,7 @@ gucharmap_window_init (GucharmapWindow *guw)
   priv->font_selection_visible = FALSE;
   priv->text_to_copy_visible = FALSE;
   priv->file_menu_visible = FALSE;
-  priv->chapters_mode = CHAPTERS_SCRIPT;
+  priv->chapters_mode = gucharmap_settings_get_chapters_mode ();
 
   priv->search_dialog = NULL;
 
